package com.iflytek.cloud;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.IdentityVerifierImpl;
import com.iflytek.msc.MSC;

/**
 * <h3>身份验证类</h3>
 * <script>gShowExtraIdentity();</script>
 * <p>
 * 		多生物特征融合验证（Multi-biometric Fusion Verification， MFV）平台是
 * <script>gShowPlatformOrCompanyNameAuto();</script>结
 * 合已有的声纹和人脸识别技术，推出的新一代身份验证平台。该平台不但可以提供单一的声
 * 纹或人脸验证，还可以进行全新的融合（人脸+声纹）验证。目前支持人脸、声纹，将来还会
 * 有指纹、虹膜等。
 * </p>
 * <p>
 * 		身份验证，主要包括注册、验证和模型操作，三个过程({@link SpeechConstant#MFV_SST});
 *  其中每个过程又分声纹、人脸和融合（声纹+人脸）业务({@link SpeechConstant#MFV_SCENES})。
 * </p>
 * <p>
 * 		注册：{@link #startWorking}开始会话前，设置{@link SpeechConstant#MFV_SST} 值
 * 为"enroll"时，指定当次会话过程为注册。注册，指由应用为用户指定一个唯一的字符串类型的授权ID
 * {@link SpeechConstant#AUTH_ID}，指定业务类型{@link SpeechConstant#MFV_SCENES}，
 * 并上传相应的用户生理特征数据，为用户在服务器中建立授权ID与特征一一对应的模型，用于用户
 * 身份验证。
 * </p>
 * <p>
 * 		验证：{@link #startWorking}开始会话前，设置{@link SpeechConstant#MFV_SST} 值
 * 为"verify"时，指定当次会话过程为验证。验证，即指定用户注册时的授权ID
 * {@link SpeechConstant#AUTH_ID}，和业务类型{@link SpeechConstant#MFV_SCENES}，
 * 并上传相应用户生理特征数据，验证是否与服务器的模型生理特征一致。
 * </p>
 * <p>
 * 		操作：在{@link #execute}前，指定用户注册时的授权ID{@link SpeechConstant#AUTH_ID}，
 * 和业务类型{@link SpeechConstant#MFV_SCENES}，并在{@link #execute}指定操作类型，
 * 可以对已存在的用户模型进行，查询、删除和下载等操作。
 * </p>
 * <p>
 * 		声纹密码：当前声纹密码，只支持在线方式，需要网络可用，方可正常使用此功能。
 * 当前声纹密码只支持汉语的随机数字串。 
 * </p>
 * <p>
 * 		随机数字串声纹密码，注册时，应用层调用{@link #execute}获取声纹密码，
 * 将由服务器随机产生2~9组各8位的随机阿拉伯数字串，如
 * “57280964-96382704-59387624”，通过{@link IdentityListener#onResult}
 * 回调给应用层。在获取密码后，应用层调用{@link #startWorking}开始注册，通过
 * {@link #writeData}把用户读
 * 密码的音频依次传给SDK，当一组说完时，SDK将回调{@link IdentityListener#onResult}
 * 通知应用层，当组密码识别的状态，应用层可根据此状态，决定继续进行下一组读密码，
 * 或者重新开始，或者结束注册。在验证时，通过 {@link com.iflytek.cloud.util.VerifierUtil#generateNumberPassword} 
 * 生成随机数字串密码后，调用{@link #startWorking}开始写入({@link #writeData})用户
 * 的音频，并通过{@link IdentityListener}获取当前验证状态。
 * </p>
 * <p>
 * 		当前声纹在随机密码方式下，支持未压缩的16位，单声道，采样率为
 * 16000，字节顺序为Little-Endian的Windows PCM音频。使用默认的参数值，或通过
 * {@link #setParameter}函数指定。
 * </p>
 * <p>
 * 		人脸：人脸识别通过对比注册和验证时，图片中人脸的生理特征，辨别是否是同一个人。
 * 人脸的注册和验证比声纹的简单，设置业务类型和操作类型后，传入一张图片数据即可。
 * </p>
 * <p>
 * 		当前人脸识别支持以下图片格式：PNG，BMP, JPG，GIF。不需要针对不同格式设置格式参数，引擎
 * 根据传入的图片数据自动辨别。
 * </p>
 * <p>
 * 		融合：即融合多种生理特征进行注册、验证。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createVerifier}创建一次对
 * 象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调用者可
 * 通过{@link #getVerifier}获取当前已经创建的单例。在销毁本类的单例对象后，需要先通过
 * {@link #createVerifier}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author<script>gShowAuthor();</script>
 * @see IdentityListener
 * @see IdentityResult
 * @see SpeechUtility#createUtility
 */
public final class IdentityVerifier extends SpeechInterface {
		
	/**
	 * 单例对象
	 */
	private static IdentityVerifier instance = null;
	
	/**
	 * 具体的Impl对象
	 */
	private IdentityVerifierImpl mVerifierImpl = null;	
	
	/**
	 * 初始化监听器
	 */
	private InitListener mInitListener = null;
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getVerifier}获取已创建的单例对象。
	 * </p>
	 * 
	 * @param context {@link android.content.Context}应用上下文对象
	 * @param listener 构造回调监听器，通过监听器获取构造状态
	 * @return 身份验证对象
	 * @see #destroy
	 * @see #getVerifier
	 */
	public synchronized static IdentityVerifier createVerifier(Context context, InitListener listener) {
		synchronized( sSync ){
			if( instance==null && null!=SpeechUtility.getUtility() ) {
				instance = new IdentityVerifier(context, listener);
			}
		}//end of sync
		
		return instance;
	}
	
	/**
	 * <h4>获取身份验证对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createVerifier}创建单例对象。
	 * </p>
	 * 
	 * @return 身份验证对象
	 * 
	 * @see #createVerifier
	 * @see #destroy
	 */
	public static IdentityVerifier getVerifier() {
		return instance;
	}
	
	/**
	 * 子类可见的构造函数。<br>
	 * @param context 上下文
	 * @param listener 初始化监听器
	 */
	protected IdentityVerifier(Context context, final InitListener listener) {
		super();
		mInitListener = listener;
		
		mVerifierImpl = new IdentityVerifierImpl(context);
		
		if(null != listener) {
			// 放在主线程中回调，修复用户在回调中立即使用本对象造成的空指针问题
//			if (!MSC.isLoaded()) {
//				Message.obtain(mUiHandler, ErrorCode.ERROR_COMPONENT_NOT_INSTALLED, 0, 0, null).sendToTarget();
//			} else {
				Message.obtain(mUiHandler, ErrorCode.SUCCESS, 0, 0, null).sendToTarget();
//			}
		}
	}
	
	private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
	
		@Override
		public void handleMessage(Message msg) {
			// 判断listener是否为null
			if(mInitListener == null) {
				return;
			}
			mInitListener.onInit(msg.what);
		}
	};

	/**
	 * <h4>开始会话</h4>
	 * <p>
	 * 		调用此函数开始注册或验证。在调用本函数开始注册或验证前，通过
	 * {@link #setParameter(String, String)}设置相应的参数。
	 * </p>
	 * <p>
	 * 		身份认证包括注册和验证。验证前需要先注册建立模型。调本此函数前，通过
	 * {@link #setParameter(String, String)}设置业务类型和过程类型。
	 * </p>
	 * <p>
	 * 		其他需要注意的地方，请参考类说明。
	 * </p>
	 * @param listener 会话监听器
	 * @return 错误码，0表示成功，请参照{@link ErrorCode}
	 * 
	 * @see #getVerifier
	 * @see #setParameter
	 * @see #writeData
	 * @see #cancel
	 */
	public int startWorking(IdentityListener listener) {
		if(mVerifierImpl == null) 
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		mVerifierImpl.setParameter(mSessionParams);
		return mVerifierImpl.startWorking(listener);
	}
	
	/**
	 * <h4>写入数据</h4>
	 * <p>
	 * 		调用此函数写入数据前，需要先调用{@link #startWorking}开始一次会话。
	 * </p>
	 * <p>
	 * 		写入的数据因业务类型而异：在声纹时，写入为音频数据；在人脸时，写入为图片数据。
	 * 关于支持的数据格式，请参考本类的介绍。
	 * </p>
	 * <p>
	 * 		可以多次调用此函数，分次把数据传给SDK。当{@link IdentityListener#onError}
	 * 回调错误，或{@link IdentityListener#onResult}
	 * 返回最后一个结果时，当次会话结束，应该停止调用此函数写入数据。
	 * </p>
	 * 
	 * @param ssub 子业务类型，可选值：ivp（声纹），ifr（人脸）
	 * @param params 子业务相关参数：<br>
	 * 		声纹业务，包括，总注册次数（rgn，值范围[2, 9]），密码（ptxt, 见类说明），密码
	 * 类型（pwdt, 值范围{3}）；<br>
	 * 		人脸业务，暂无。
	 * @param buffer 数据源<br>
	 * @param offset buffer有效数据开始位置<br>
	 * @param length buffer有效数据长度<br>
	 * 
	 * @return 错误码，0表示成功，其他则表示有错误，参考{@link ErrorCode}。
	 * 
	 * @see #startWorking
	 * @see IdentityListener
	 * @see #setParameter
	 */
	public int writeData(String ssub, String params, byte[] buffer, int offset, 
			int length) {
		if (null != mVerifierImpl && mVerifierImpl.isWorking()) {
			DebugLog.LogE("mVerifierImpl is" + (null != mVerifierImpl) + "  mVerifierImpl.isWorking() is  " + mVerifierImpl.isWorking());
			return mVerifierImpl.writeData(ssub, params, buffer, offset, length);
		}else {
			DebugLog.LogE("IdentityVerifier writeAudio failed, is not running");
			return ErrorCode.ERROR_ENGINE_CALL_FAIL;
		}
	}
	
	/**
	 * <h4>停止写入数据</h4>
	 * <p>
	 * 		调用此函数，告知SDK数据写入已完成。停止写入数据后，结果不能即时返回，视网络
	 * 情况等而异，一般在1秒内可通过{@link IdentityListener#onResult}获取到结果。当
	 * {@link IdentityListener#onError}回调会话错误时，会话结束，将无结果返回。
	 * </p>
	 * <p>
	 * 		在当次会话，已回调{@link IdentityListener#onError}
	 * 或者{@link IdentityListener#onResult}的最后一个结果时，会话自动结束，可不再调用此函数告知
	 * SDK数据写入完成。
	 * </p>
	 * @param ssub 子业务类型，与writeData中的ssub对应，可选值：ivp（声纹），ifr（人脸）
	 * 
	 * @see #startWorking
	 * @see #cancel
	 * @see #writeData
	 */
	public void stopWrite(String ssub) {
		if(null != mVerifierImpl && mVerifierImpl.isWorking()) {
			mVerifierImpl.stopWrite(ssub);
		} else {
			DebugLog.LogE("IdentityVerifier stopListening failed, is not running");
		}
	}
	
	/**
	 * <h4>模型操作</h4>
	 * <p>
	 * 		模型操作，包括查询，删除，下载。通过{@link IdentityListener}获取操作的状态和结果。
	 * </p>
	 * <p>
	 * 		模型操作和开始会话函数{@link #startWorking}是两个不同的功能函数，执行此函数前，
	 * 不需要调用{@link #startWorking}开始会话。
	 * </p>
	 * <p>
	 * 		调用此函数后，操作结果不能即时返回，视网络
	 * 情况等而异，一般在1秒内可通过{@link IdentityListener#onResult}获取到结果。当
	 * {@link IdentityListener#onError}回调会话错误时，会话结束，将无结果返回。
	 * </p>
	 * 
	 * @param ssub 子业务类型，可选值：ivp，ifr，ipt。
	 * @param cmd 操作命令，可选值：query（模型查询），delete（模型删除），download
	 * 		（密码下载，暂时只用于声纹业务），add (添加组)。
	 * @param params cmd参数。
	 * @param listener 结果监听器。
	 * @return 状态， 当返回{@link ErrorCode#SUCCESS}时，表示正常开始操作；否则，开始操作失败。
	 * 
	 * @see #startWorking
	 * @see IdentityListener
	 */
	public int execute(String ssub, String cmd, String params, IdentityListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		if(mVerifierImpl != null) {
			if( mVerifierImpl.setParameter(mSessionParams) ){
				ret = mVerifierImpl.execute(ssub, cmd, params, listener);
			}else{
				ret = ErrorCode.ERROR_INVALID_PARAM;
			}
		} else {
			DebugLog.LogE("IdentityVerifier execute failed, is not running");
			ret = ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		}
		
		return ret;
	}
	
	/**
	 * <h4>是否正在会话</h4>
	 * <p>
	 * 		获取当前是否正在会话。如果当前调用{@link #startWorking}或{@link #execute}后，
	 * {@link IdentityListener}没有通过{@link IdentityListener#onError}返回错误，或
	 * {@link IdentityListener#onResult}返回最后一个结果时，当前正在会话中。反之，当前不
	 * 在会话中。（注意当{@link IdentityListener#onResult}返回最后一个结果时，会话结束可
	 * 能有短暂的延时。）
	 * </p>
	 * 
	 * @return false：否；true：是。
	 * 
	 * @see #startWorking
	 * @see #execute
	 * @see #cancel
	 */
	public boolean isWorking() {
		if (null != mVerifierImpl && mVerifierImpl.isWorking()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * <h4>取消会话</h4>
	 * <p>
	 * 		取消当前会话。会话被取消后，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #startWorking
	 * @see #execute
	 * @see #stopWrite
	 */
	public void cancel() {
		if (null != mVerifierImpl && mVerifierImpl.isWorking()) {
			mVerifierImpl.cancel(false);
		} else {
			DebugLog.LogE("IdentityVerifier cancel failed, is not running");
		}
	}

	 /**
	 * <h4>参数设置</h4>
	 * <p>
	 * 		可设置的参数有：<br>
	 * {@link SpeechConstant#SUBJECT}：业务类型，可选值：mfv，ivp，ifr。<br>
	 * {@link SpeechConstant#MFV_SCENES}：场景，mfv业务必须传此参数。<br>
	 * {@link SpeechConstant#MFV_SST}：会话类型。<br>
	 * {@link SpeechConstant#MFV_VCM}：验证模式，仅在融合验证场景下使用<br>
	 * {@link SpeechConstant#MFV_AFC}：确认周期(affirmance cycle)，仅在灵活融合验证场景下使用。
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型（目前身份认证仅支持在线模式）；
	 * 
	 * @param key   参数名称。
	 * @param value 参数值。
	 * @return true 设置成功，否则设置失败。
	 * 
	 * @see #startWorking
	 * @see #getParameter
	 */
	@Override
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}
	
	/**
	 * <h4>获取参数</h4>
	 * <p>参考{@link #setParameter}。</p>
	 * 
	 * @param key 参数名称。<br>
	 * @return 参数值。
	 * 
	 * @see #setParameter
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}

	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createVerifier}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isWorking}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createVerifier}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createVerifier
	 * @see #cancel
	 */
	public boolean destroy() {
		boolean ret = true;
		final IdentityVerifierImpl impl = mVerifierImpl;
		if( impl != null ) {
			ret = impl.destroy();
		}
		
		if( ret && (ret=super.destroy()) ) {
			synchronized( sSync ){	
				instance = null;
			}//end of sync
		}
		
		return ret;
	}
}
