/*
 * 文件名：SpeechSynthesizer.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：合成对象，包括开始合成、取消合成、暂停合成、恢复合成等接口；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.RemoteException;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.speech.impl.SpeechSynthesizerImpl;
import com.iflytek.cloud.util.ResourceUtil;
import com.iflytek.msc.MSC;
import com.iflytek.speech.SpeechSynthesizerAidl;

/**
 * <h3>语音合成类</h3>
 * <p>
 * 		语音合成，又称文语转换（Text to Speech，TTS）技术，解决的主要问题
 * 是如何将文字信息转化为可听的声音信息（即音频数据）。关于语音识别，请
 * 参考{@link SpeechRecognizer}类。
 * </p>
 * <p>
 * 		语音合成，支持不同的语言，方言，或同一方言不同的人的声音特点等，请参考
 * 发音人参数{@link SpeechConstant#VOICE_NAME}。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createSynthesizer}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getSynthesizer()}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createSynthesizer}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility#createUtility
 * @see SpeechRecognizer
 */
public class SpeechSynthesizer extends SpeechInterface{
	
	/**
	 * 合成单例对象
	 */
	private static SpeechSynthesizer sInstance = null;
	
	/**
	 *  客户端插件MSC合成实例
	 */
	private SpeechSynthesizerImpl mSynthesizer = null;
	
	/**
	 *  语记合成实例
	 */
	private SpeechSynthesizerAidl mSynthesizerAidl = null;
	
	/**
	 *  语记合成监听
	 */
	private InternalListener mInternalAidlListener = null;
	
	/**
	 * 初始化回调监听
	 */
	InitListener mInitListener = null;
	
	/**
	 * 内部识别监听类，保证回调在主线程完成
	 * 
	 *
	 */
	private final class InternalListener implements SynthesizerListener{
		private SynthesizerListener mOutListener = null;
		private com.iflytek.speech.SynthesizerListener mAidlListener = null;
		
		private static final int MSG_BEGIN = 1;
		private static final int MSG_BUFFER_PERCENT = 2;
		private static final int MSG_PAUSE = 3;
		private static final int MSG_RESUME = 4;
		private static final int MSG_PLAY_PERCENT = 5;
		private static final int MSG_COMPLETE = 6;
		private static final int MSG_EVENT = 7;
		
		private static final String KEY_PERCENT = "percent";
		private static final String KEY_BEGPOS = "begpos";
		private static final String KEY_ENDPOS = "endpos";
		private static final String KEY_SPELL_INFO = "spellinfo";
		
		public InternalListener(SynthesizerListener outListener) {
			mOutListener = outListener;
			mAidlListener = new com.iflytek.speech.SynthesizerListener.Stub() {
				
				@Override
				public void onSpeakResumed() throws RemoteException {
					if(null != mOutListener)
						Message.obtain(mUiHandler, MSG_RESUME, 0, 0, null).sendToTarget();
				}
				
				@Override
				public void onSpeakProgress(int progress, int beginPos,
						int endPos) throws RemoteException {
					if(null != mOutListener)
						Message.obtain(mUiHandler, MSG_PLAY_PERCENT, progress, beginPos, endPos).sendToTarget();
				}
				
				@Override
				public void onSpeakPaused() throws RemoteException {
					if( null != mOutListener)
						Message.obtain(mUiHandler, MSG_PAUSE).sendToTarget();
				}
				
				@Override
				public void onSpeakBegin() throws RemoteException {
					if( null != mOutListener)
						Message.obtain(mUiHandler, MSG_BEGIN).sendToTarget();
				}
				
				@Override
				public void onCompleted(int errorCode) throws RemoteException {
					if( null != mOutListener)
						Message.obtain(mUiHandler, MSG_COMPLETE, (ErrorCode.SUCCESS == errorCode ? null : new SpeechError(errorCode))).sendToTarget();
				}
				
				@Override
				public void onBufferProgress(int progress, int beginPos,
						int endPos, String info) throws RemoteException {
					if( null != mOutListener)
					{
						Bundle bundle = new Bundle();
						bundle.putInt(KEY_PERCENT, progress);
						bundle.putInt(KEY_BEGPOS, beginPos);
						bundle.putInt(KEY_ENDPOS, endPos);
						bundle.putString(KEY_SPELL_INFO, "");

						if (null != mOutListener)
							Message.obtain(mUiHandler, MSG_BUFFER_PERCENT, bundle).sendToTarget();
						
					}
				}

				@Override
				public void onEvent(int eventType, int arg1, int arg2,
						Bundle obj) throws RemoteException {
					if(null != mOutListener)
					{
						Message msg = Message.obtain();
						msg.what = eventType;
						msg.arg1 = 0;
						msg.arg2 = 0;
						msg.obj = obj;
						Message.obtain(mUiHandler, MSG_EVENT, 0, 0,  msg).sendToTarget();
					}
				}

			};
		}
		
		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				//listener 判断是否为null
				if(mOutListener == null)
					return;
				
				switch (msg.what) {
				case MSG_BEGIN:
					mOutListener.onSpeakBegin();
					break;
				case MSG_BUFFER_PERCENT:
				{
					Bundle bundle = (Bundle)msg.obj;
					int percent = bundle.getInt(KEY_PERCENT);
					int begpos = bundle.getInt(KEY_BEGPOS);
					int endpos = bundle.getInt(KEY_ENDPOS);
					String spell = bundle.getString(KEY_SPELL_INFO);
					mOutListener.onBufferProgress(percent, begpos, endpos, spell);
				}
					break;
				case MSG_PAUSE:
					mOutListener.onSpeakPaused();
					break;
				case MSG_RESUME:
					mOutListener.onSpeakResumed();
					break;
				case MSG_PLAY_PERCENT:
				{	
					int endPos = (Integer)msg.obj;						
					mOutListener.onSpeakProgress(msg.arg1, msg.arg2, endPos);
				}
				break;
				case MSG_COMPLETE:
					mOutListener.onCompleted((SpeechError)msg.obj);
					break;
				case MSG_EVENT:
					Message event = (Message) msg.obj;
					if(event != null)
					{
						mOutListener.onEvent(event.what, event.arg1, event.arg2, (Bundle)event.obj);
					}
					break;
				default:
					break;
				}
			}
		};

		@Override
		public void onSpeakBegin() {
			if( null != mOutListener)
				Message.obtain(mUiHandler, MSG_BEGIN).sendToTarget();
		}

		@Override
		public void onBufferProgress(int progress, int beginPos, int endPos,
				String info) {
			if( null != mOutListener)
			{
				Bundle bundle = new Bundle();
				bundle.putInt(KEY_PERCENT, progress);
				bundle.putInt(KEY_BEGPOS, beginPos);
				bundle.putInt(KEY_ENDPOS, endPos);
				bundle.putString(KEY_SPELL_INFO, info);

				if (null != mOutListener)
					Message.obtain(mUiHandler, MSG_BUFFER_PERCENT, bundle).sendToTarget();
				
			}
		}

		@Override
		public void onSpeakPaused() {
			if( null != mOutListener)
				Message.obtain(mUiHandler, MSG_PAUSE).sendToTarget();
			
		}

		@Override
		public void onSpeakResumed() {
			if( null != mOutListener)
				Message.obtain(mUiHandler, MSG_RESUME).sendToTarget();
			
		}

		@Override
		public void onSpeakProgress(int progress, int beginPos, int endPos) {
			if(null != mOutListener)
				Message.obtain(mUiHandler, MSG_PLAY_PERCENT, progress, beginPos, endPos).sendToTarget();
			
		}

		@Override
		public void onCompleted(SpeechError error) {
			if( null != mOutListener)
				Message.obtain(mUiHandler, MSG_COMPLETE, error).sendToTarget();
			
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			if( null != mOutListener)
			{			
				Message msg = Message.obtain();
				msg.what = eventType;
				msg.arg1 = 0;
				msg.arg2 = 0;
				msg.obj = obj;
				Message.obtain(mUiHandler, MSG_EVENT, 0, 0,  msg).sendToTarget();		
			}
		}
		
	}
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getSynthesizer()}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 合成对象
	 * @see #destroy
	 * @see #getSynthesizer
	 */
	public static SpeechSynthesizer createSynthesizer(Context context,InitListener listener) {
		
		synchronized( sSync ){
			if( sInstance==null && null!=SpeechUtility.getUtility() ){
				sInstance = new SpeechSynthesizer(context,listener);
			}
		}//end of sync
		
		return sInstance;
	}

	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createSynthesizer}创建单例对象。
	 * </p>
	 * 
	 * @return 合成对象
	 * 
	 * @see #createSynthesizer
	 * @see #destroy
	 */
	public static SpeechSynthesizer getSynthesizer() {
		return sInstance;
	}

	protected SpeechSynthesizer(Context context, InitListener listener) {
		super();
		
		mInitListener = listener;
		
		// 分别创建com.iflytek.speech.SpeechSynthesizerImpl及com.iflytek.speech.SpeechSynthesizerAidl
		mSynthesizer = new SpeechSynthesizerImpl(context);
		
		//创建语记识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
			&& utility.getEngineMode() != ENGINE_MODE.MSC) 
		{
			mSynthesizerAidl = new com.iflytek.speech.SpeechSynthesizerAidl(context.getApplicationContext(), listener);
		}
		else
		{
			//放在主线程中回调，修复用户在回调中立即使用本对象造成的空指针问题
			Message.obtain(mUiHandler, ErrorCode.SUCCESS, 0, 0, null).sendToTarget();
		}
	}
	
	private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
		@Override
		public void handleMessage(Message msg) {
			//listener 判断是否为null
			if(mInitListener == null)
				return;
			mInitListener.onInit(ErrorCode.SUCCESS);
		}
	};
	
	protected void reInit(Context context) 
	{
		//创建语记识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
				&& utility.getEngineMode() != ENGINE_MODE.MSC) 
		{
			//判断服务是否可用，在用户升级语记造成的服务端开时会出现服务不可用的情况
			if(null != mSynthesizerAidl && !mSynthesizerAidl.isAvailable())
			{
				mSynthesizerAidl.destory();
				mSynthesizerAidl = null;
			}
			mSynthesizerAidl = new com.iflytek.speech.SpeechSynthesizerAidl(context.getApplicationContext(), mInitListener);
		}
		else if(null != mInitListener) 
		{
			if(null != mSynthesizerAidl)
			{
				mSynthesizerAidl.destory();
				mSynthesizerAidl = null;
			}
		}
	}
	
	/**
	 * <h4>开始合成</h4>
	 * <p>
	 * 		调用此函数，开始合成文本并播放音频。关于合成到文件，请参考函数{@link #synthesizeToUri}。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话。一次会话，即从会话开始，到结
	 * 束为止。
	 * </p>
	 * <p>
	 * 		在调用本函数开始听写或语法识别前，通过{@link #setParameter(String, String)}设置相
	 * 应的参数。
	 * </p>
	 * <p>
	 * 		可通过{@link #stopSpeaking()}取消当前的会话。
	 * </p>
	 * 
	 * @param text 待合成的文本
	 * @param listener 合成状态监听器
	 * 
	 * @see #createSynthesizer
	 * @see #getSynthesizer()
	 * @see #synthesizeToUri
	 * @see #stopSpeaking()
	 * @see #setParameter(String, String)
	 * @see SynthesizerListener
	 */
	public int startSpeaking(String text, SynthesizerListener listener)
	{
		DebugLog.LogD( "stop all current session in new session" );
		stopSpeaking();
		
		ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_TTS, mSynthesizerAidl);
		//@{ start: 去除语记支持
		//有安装语记
		/*if(mode == ENGINE_MODE.PLUS)
		{
			if(mSynthesizerAidl == null)
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mSynthesizerAidl.setParameter(SpeechConstant.PARAMS, null);
			mSynthesizerAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
			mSessionParams.removeParam( SpeechConstant.NEXT_TEXT );
			//在主线程抛出回调消息
			mInternalAidlListener = new InternalListener(listener);
			
			return mSynthesizerAidl.startSpeaking(text, mInternalAidlListener.mAidlListener);
		}
		else
		{
			if(mSynthesizer == null)
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mSynthesizer.setParameter(mSessionParams);
			mSessionParams.removeParam( SpeechConstant.NEXT_TEXT );
			return mSynthesizer.startSpeaking(text, listener);
		}*/
		//@ end: 去除语记支持}
		if(mSynthesizer == null)
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		mSynthesizer.setParameter(mSessionParams);
		mSessionParams.removeParam( SpeechConstant.NEXT_TEXT );
		return mSynthesizer.startSpeaking(text, listener);
	}
	
	/**
	 * <h4>合成到文件</h4>
	 * <p>
	 * 		合成文本到一个音频文件，不播放。使用此函数时，请考虑应用是否
	 * 有在设置的目录中保存文件的权限。
	 * </p>
	 * 
	 * @param text 要合成的文本
	 * @param uri 要保存的文件路径（含文件名）
	 * @param listener 合成音频回调接口
	 *            
	 *  @see #startSpeaking(String, SynthesizerListener)
	 */
	public int synthesizeToUri(String text,String uri, SynthesizerListener listener)
	{
		//@{ start:去除语记支持
		/*ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_TTS, mSynthesizerAidl);
		if( mode == ENGINE_MODE.PLUS)
		{
			if(mSynthesizerAidl == null)
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mSynthesizerAidl.setParameter(SpeechConstant.PARAMS, null);
			mSynthesizerAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
			mSynthesizerAidl.setParameter(MscKeys.TTS_AUDIO_URI, uri);
			mInternalAidlListener = new InternalListener(listener);
			return mSynthesizerAidl.synthesizeToUrl(text, mInternalAidlListener.mAidlListener);
		}else if (null != mSynthesizer)
		{
			mSynthesizer.setParameter(mSessionParams);
			return mSynthesizer.synthesizeToUri(text, uri, listener);
		}*/
		if(null!= mSynthesizer) {
			mSynthesizer.setParameter(mSessionParams);
			return mSynthesizer.synthesizeToUri(text, uri, listener);
		}
		//@ end: 去除语记支持}
		return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
	}
	
	/**
	 * <h4>暂停播放</h4>
	 * <p>
	 * 		暂停播放，仅在合成播放模式下有效，暂停播放，并不会暂停音频的获取过程，
	 * 只是在把播放器暂停。
	 * </p>
	 * 
	 * @see #resumeSpeaking()
	 * @see #startSpeaking(String, SynthesizerListener)
	 */
	public void pauseSpeaking()
	{
		if (null != mSynthesizer && mSynthesizer.isSpeaking())
		{
			mSynthesizer.pauseSpeaking();
		}
		else if( null != mSynthesizerAidl && mSynthesizerAidl.isSpeaking())
		{
			if(mInternalAidlListener != null)
			{
				mSynthesizerAidl.pauseSpeaking(mInternalAidlListener.mAidlListener);
			}
		}
	}
	
	/**
	 * <h4>恢复播放</h4>
	 * <p>
	 * 		在暂停后，在当前暂停位置开始播放合成的音频。
	 * </p>
	 * 
	 * @see #pauseSpeaking()
	 * @see #stopSpeaking()
	 */
	public void resumeSpeaking()
	{
		if (null != mSynthesizer && mSynthesizer.isSpeaking())
		{
			mSynthesizer.resumeSpeaking();
		}
		else if( null != mSynthesizerAidl && mSynthesizerAidl.isSpeaking())
		{
			if(mInternalAidlListener != null)
			{
				mSynthesizerAidl.resumeSpeaking(mInternalAidlListener.mAidlListener);
			}
		}
		
	}
	
	/**
	 * <h4>停止合成</h4>
	 * <p>
	 * 		调用此函数，取消当前合成会话，并停止音频播放。调用此函数后，未合
	 * 成的音频将不再返回，若为合成到文件模式，音频文件将不会被保存。
	 * </p>
	 * 
	 * @see #startSpeaking(String, SynthesizerListener)
	 */
	public void stopSpeaking()
	{
		if (null != mSynthesizer && mSynthesizer.isSpeaking())
		{
			mSynthesizer.stopSpeaking(false);
		}
		
		if( null != mSynthesizerAidl && mSynthesizerAidl.isSpeaking())
		{
			if(mInternalAidlListener != null)
			{
				mSynthesizerAidl.stopSpeaking(mInternalAidlListener.mAidlListener);
			}
		}
	}
	
	/**
	 * <h4>是否在合成</h4>
	 * <p>
	 * 		是否在合成状态，包括是否在播放状态，音频从服务端获取完成后，若未播放
	 * 完成，依然处于当前会话的合成中。
	 * </p>
	 * 
	 * @return true 是，false 否
	 * 
	 * @see #startSpeaking(String, SynthesizerListener)
	 * @see #stopSpeaking()
	 */
	public boolean isSpeaking()
	{
		if (null != mSynthesizer && mSynthesizer.isSpeaking())
		{
			return true;
		}
		else if( null != mSynthesizerAidl && mSynthesizerAidl.isSpeaking())
		{
			return true;
		}
		
		return false;
	}
	 
	/**
	 * <h4>参数设置</h4>
	 * <p>
	 *		可设置的参数有：<br>
	 * {@link SpeechConstant#VOICE_NAME}: 发音人<br>
	 * {@link SpeechConstant#SPEED}: 合成语速<br>
	 * {@link SpeechConstant#VOLUME}: 合成音量<br>
	 * {@link SpeechConstant#PITCH}: 合成语调<br>
	 * {@link SpeechConstant#BACKGROUND_SOUND}: 背景音乐<br>
	 * {@link SpeechConstant#TTS_BUFFER_TIME}: 合成音频缓冲时间<br>
	 * {@link SpeechConstant#STREAM_TYPE}: 播放类型<br>
	 * {@link SpeechConstant#SAMPLE_RATE}: 采样率<br>
	 * {@link SpeechConstant#TTS_AUDIO_PATH}: 合成录音保存路径
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型；
	 * <br>{@link com.iflytek.cloud.util.ResourceUtil#TTS_RES_PATH}：离线资源路径；
	 * <br>{@link com.iflytek.cloud.util.ResourceUtil#ENGINE_START}：启动离线引擎；
	 * <br>{@link SpeechConstant#TTS_FADING} : 合成淡入淡出；
	 * <br>{@link SpeechConstant#AUDIO_FORMAT_AUE}:音频流编解码格式；
	 *
	 * <p>
	 * 
	 * @param key 参数名
	 * @param value 参数值
	 * @return 设置是否成功
	 * 
	 * @see #startSpeaking(String, SynthesizerListener)
	 * @see #getParameter(String)
	 */
	@Override
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}
	

	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#VOICE_NAME}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key
	 *            参数名称，所有支持参数，参考{@link #setParameter(String, String)}
	 * @return 参数值，值意义，参考{@link SpeechConstant}对各参数的说明。
	 * 
	 * @see #setParameter(String, String)
	 */
	@Override
	public String getParameter(String key) 
	{
		// 本地发音人获取，需要单独执行
		if(SpeechConstant.LOCAL_SPEAKERS.equals(key) && mSynthesizerAidl != null)
		{
			return mSynthesizerAidl.getParameter(key);
		}
		// 获取播放状态
		else if (SpeechConstant.TTS_PLAY_STATE.equals(key))
		{
			//@{ start: 去除语记支持
			/*ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_TTS, mSynthesizerAidl);
			//有安装语记
			if(mode == ENGINE_MODE.PLUS  && mSynthesizerAidl != null)
			{
				return mSynthesizerAidl.getParameter(key);
			}
			else */
			//@ end: 去除语记支持}
			if (mSynthesizer != null)
			{
				return ""+mSynthesizer.getState();
			}
		}
		return super.getParameter(key);
	}

	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createSynthesizer}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isSpeaking()}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createSynthesizer}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createSynthesizer
	 * @see #stopSpeaking()
	 */
	public boolean destroy() {
		boolean ret = true;
		
		final SpeechSynthesizerAidl aidl = this.mSynthesizerAidl;
		if(null != aidl){
			aidl.destory();
		}
		
		final SpeechSynthesizerImpl  impl = this.mSynthesizer;
		if(impl != null){
			ret = impl.destroy();
		}
		
		if( ret && (ret=super.destroy()) ){
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
			
			//销毁离线引擎和资源
			final SpeechUtility utility = SpeechUtility.getUtility();
			if( null != utility ){
				DebugLog.LogD( "Destory tts engine." );
				utility.setParameter( ResourceUtil.ENGINE_DESTROY, ResourceUtil.ENGINE_DESTROY+MscKeys.KEY_VAL_SEP+SpeechConstant.ENG_TTS );
			}
		}
		
		return ret;
	}
	
}
