package com.iflytek.cloud.record;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import android.media.MediaRecorder;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * PCM Recoder
 * 
 * @author iflytek
 * 
 */
public class FilePcmRecorder extends PcmRecorder{
	
	//每次从文件中读取的数据长度
	private int mReadLen = 0;
	//读取的数据偏移量
	private int mOffset = 0;
	//40ms录音数据长度
	private int mRecordLen = 0;
	private boolean exit = false;
	
	private int mRate = PcmRecorder.RATE16K;
	public static final int DEF_RECORD_INTERVAL = 40;
	private final short DEFAULT_BIT_SAMPLES = 16;
	private static final short DEFAULT_CHANNELS = 1;
	
	// 录音回调频度，外部传入间隔
	private int mInterval = DEF_RECORD_INTERVAL;
	// 读取录音频度，线程中使用
	private int mReadInterval = DEF_RECORD_INTERVAL;

	private byte[] mDataBuffer = null;
	private RandomAccessFile mRecordFile = null;
	private String mRecordFileName = null;
	private PcmRecordListener mOutListener = null;
	/**
	 * 
	 * @throws Exception
	 */
	public FilePcmRecorder(int sampleRate,int timeInterval, String filepath) {
		this(sampleRate, timeInterval, MediaRecorder.AudioSource.MIC, filepath);
	}
	
	public FilePcmRecorder(int sampleRate,int timeInterval,int audioSource, String filepath) {
		super(sampleRate, timeInterval, audioSource);
		
		mRate = sampleRate;
		mInterval = timeInterval;
		mReadInterval = timeInterval;
		mRecordFileName = filepath;
	}

	/**
	 * 
	 * @param channels 通道
	 * @param sampleRate 采样率
	 * @throws SpeechError
	 */
	@Override
	protected void initRecord(short channels, int sampleRate,int timeInterval)
			throws SpeechError {

		short bitSamples = DEFAULT_BIT_SAMPLES;
		int framePeriod = sampleRate * 40 / 1000;
		mRecordLen = framePeriod * channels * bitSamples / 8;
		//每次读取的10个40ms的数据，减小io操作的频率
		mDataBuffer = new byte[ 10 * mRecordLen];
		
		try {
			mRecordFile = new  RandomAccessFile(mRecordFileName, "r");
		} catch (FileNotFoundException e) {
			throw new SpeechError(ErrorCode.ERROR_AUDIO_RECORD);
		}
	}

	/**
	 * @throws SpeechError 
	 * @throws IOException
	 * 
	 */
	private int readRecordData() throws SpeechError {
		int count = 0;
		if (mRecordFile == null || mOutListener == null)
			return 0;
		
		//buffer中数据都抛出了，需要从文件中读取数据
		if(mOffset >= mReadLen)
		{
			try {
				mOffset = 0;
				mReadLen = mRecordFile.read(mDataBuffer, mOffset, mDataBuffer.length);
				if(mReadLen < 0)
				{
					return -1;
				}
					
			} catch (IOException e) {
				//DebugLog.LogE(e);
				//抛出录音错误
				throw new SpeechError(ErrorCode.ERROR_AUDIO_RECORD);
			}
		}

		if (mReadLen > 0 && mOutListener != null) 
		{
			//从mDataBuffer返回不超过mFramLen的长度的音频
			count =( (mReadLen - mOffset) > mRecordLen) ? mRecordLen : (mReadLen - mOffset) ;
			mOutListener.onRecordBuffer(mDataBuffer, mOffset, count);
			mOffset += count;
		}
		return count;
	}
	
	
	@Override
	public void stopRecord(boolean forceStop) {
		exit = true;
	}

	/**
	 * @throws SpeechError 
	 * 
	 */
	public void startRecording(PcmRecordListener listener) throws SpeechError
	{
		mOutListener = listener;
		setPriority(MAX_PRIORITY);
		start();
	}
	
	@Override
	public void run() 
	{
		try {
			initRecord(DEFAULT_CHANNELS, mRate,mInterval);
			
			if(null != mOutListener)
				mOutListener.onRecordStarted(true);

			while(!exit)
			{
				int ret = readRecordData();
				if(ret < 0 )
				{
					exit = true;
					break;
				}
				sleep(mReadInterval);
			}
		} catch (Exception e) {
			DebugLog.LogE(e);
			if(mOutListener != null)
				mOutListener.onError(new SpeechError(ErrorCode.ERROR_AUDIO_RECORD));
		}
		release();
	}
	
	@Override
	protected void finalize() throws Throwable {
		release();
		super.finalize();
	}
	/**
	 * 
	 */
	private void release() {
		if (mRecordFile != null) {
			DebugLog.LogD("release record begin");
			try {
				mRecordFile.close();
			} catch (IOException e) {
				DebugLog.LogE(e);
			}
			mRecordFile = null;
			if(mOutListener != null)
			{
				mOutListener.onRecordReleased();
				mOutListener = null;
			}
			
			DebugLog.LogD("release record over");
		}
		if(null != mDataBuffer)
			mDataBuffer = null;
	}
}
