package com.iflytek.cloud.speech.impl;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.util.VerifierUtil;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

/**
 * 离线人脸功能实现类
 * 
 * @author jianhuang3
 * @date 2014年12月16日 上午11:37:51
 */
public class FaceDetectorImpl {
	private boolean mDestroyed;
	private MSCSessionInfo info;
	
	public FaceDetectorImpl(Context context, String param) {
		try {
			SpeechUtility utility = SpeechUtility.getUtility();
			if (null == utility) {
				throw new SpeechError(ErrorCode.MSP_ERROR_NOT_INIT);
			}
			mDestroyed = false;
			initResource(param);
		} catch (SpeechError e) {
			DebugLog.LogE(e.getPlainDescription(true));
		}
	}
	
	/**
	 * 初始化资源
	 * */
	private void initResource(String param) {
		if(MSC.isLoaded()) {
			DebugLog.LogD("MSC isLoaded："+MSC.isLoaded());
			info = new MSCSessionInfo();
			PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
			if(!TextUtils.isEmpty(param)) {
				MSC.QIFDInit(param.getBytes(), info);
			} else {
				MSC.QIFDInit(null, info);
			}
			PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
			if (ErrorCode.SUCCESS != info.errorcode) {
				DebugLog.LogE("QIFDINIT INIT FAIL, ERRORCODE:"+info.errorcode);
			} else {
				DebugLog.LogD("QIFDINIT INIT SUCCESS");
			}
		}
	}
	
	/**
	 * 检测ARGB图片中的人脸
	 * 
	 * @param src 人脸图片
	 * @return 人脸数组，没有检测到人脸则返回空
	 * */
	public String detectARGB(Bitmap src) {
		if (null != src && Config.ARGB_8888.equals(src.getConfig())) {
			Bitmap grayBmp = VerifierUtil.ARGB2Gray(src);
			return detectGray(grayBmp);
		} else {
			DebugLog.LogE("Method detectARGB:null parameter or not ARGB bitmap");
			return null;
		}
	}
		
	/**
	 * 检测灰度图片中的人脸
	 * 
	 * @param src 灰度人脸图片
	 * @return 人脸数组，没有检测到人脸则返回空
	 */
	public String detectGray(Bitmap src) {
		if (null == src || !Config.ALPHA_8.equals(src.getConfig())) {
			DebugLog.LogE("Method detectGray:null parameter or not gray bitmap");
			return null;
		}
		info = new MSCSessionInfo();
		PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		byte [] res = MSC.QIFDFacedetect(src, VerifierUtil.getBitmapsize(src), info);
		PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);

		return addErrorInfo(res, info);
	}
	
	/**
	 * 检测nv21视频帧中的人脸
	 * 
	 * @param nv21 视频帧数据
	 * @param width 视频帧图像宽，暂时只支持640
	 * @param height 视频帧图像高，暂时只支持480
	 * @param dir 图像的方向
	 * @return 人脸数组，没有检测到人脸则返回空
	 * */
	public String trackNV21(byte[] nv21, int width, int height, int align, int dir) {
		if (null == nv21 || width <= 0 || height <= 0 || dir < 0 || dir > 3) {
			DebugLog.LogE("Method trackNV21:invalid parameters");
			return null;
		}
		// 启动视频流检测
		info = new MSCSessionInfo();
		PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		byte[] result = MSC.QIFDMultitracker(nv21, nv21.length, width, height, align, dir, info);
		PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
		
		return addErrorInfo(result, info);
	}
	
	/**
	 * 在返回结果中添加
	 * @param data
	 * @param info
	 * @return
	 */
	private String addErrorInfo(byte[] data, MSCSessionInfo info) {
		JSONObject result = new JSONObject();
		try {
			if (ErrorCode.SUCCESS == info.errorcode) {
				String rst = new String(data);
				result = new JSONObject(rst);
				
			} 
			result.put("ret", info.errorcode);
		} catch (JSONException e) {
			DebugLog.LogE("face result add errorinfo exception");
		}
		return result.toString();
	}
	
	
//	/**
//	 * 获取视频帧预处理缓存
//	 * */
//	public NativeImageBuffer getTrackPreprocessBuffer() {
//		return FaceSDK.getPreprocessesBuffer();
//	}
	
	/**
	 * 销毁资源
	 * */
	public void destroy() {
		if (!mDestroyed) {
			DebugLog.LogD("QIFDFINIT");
			PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
			int ret = MSC.QIFDFini();
			DebugLog.LogD("MSC.QIFDFini result is " + ret);
			PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);
			mDestroyed = true;
		}
	}
}
