package com.iflytek.cloud.speech.impl;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.IdentityListener;
import com.iflytek.cloud.IdentityResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.mfv.MfvVerifier;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * 身份验证实现类，对外不可见。<br>
 * 
 * @author iFlytek &nbsp;&nbsp;&nbsp;<a href="http://www.xfyun.cn/">讯飞语音云</a>
 */
public class IdentityVerifierImpl extends SpeechModule {

    // 是否会话过程中暂停后台音乐播放
    private boolean mRequestFocus = false;

    public IdentityVerifierImpl(final Context context) {
        super(context);
    }

    /**
     * 性能结构化日志插入
     * 
     * @param key
     */
    public void pushSessionInfo(final String key) {
        synchronized (mSynObj) {
            if (null != mscer) {
                ((MfvVerifier) mscer).getPerfLog().pushSessionInfo(key);
            }
        }
    }

    public int startWorking(final IdentityListener listener) {
    	int ret = ErrorCode.SUCCESS;
    	
        synchronized (mSynObj) {
        	try{
                mRequestFocus = mSessionParams.getBoolean(
                        SpeechConstant.KEY_REQUEST_FOCUS, true);
    			if (null != mscer && mscer.isRunning()) {
    				// 判断是否通知外部识别被异常打断
    				mscer.cancel(mSessionParams.getBoolean(
    						SpeechConstant.MFV_INTERRUPT_ERROR, false));
    			}
                
                mscer = new MfvVerifier(mContext, mSessionParams, getHandlerThread("mfv"));
                // 识别会话开始，暂停后台音乐播放.
                FuncAdapter.Lock(mContext, mRequestFocus, null);
                ((MfvVerifier) mscer).startWorking(new InternalListener(listener));
        	}catch(SpeechError e){
    			ret = e.getErrorCode();
    			DebugLog.LogE( e );
    		}catch( Throwable e ){
    			ret = ErrorCode.ERROR_UNKNOWN;
    			DebugLog.LogE( e );
    		}//end of try-catch
        }//end of synchronized
        
        return ret;
    }

    public int writeData(final String ssub, final String params, final byte[] buffer, final int offset,
            final int length) {
        synchronized (mSynObj) {
            if (mscer == null) {
                DebugLog.LogE("writeAudio error, no active session.");
                return ErrorCode.ERROR_ENGINE_CALL_FAIL;
            }
            
            if (length < 0) {
                DebugLog.LogE("writeAudio error, length < 0.");
                return ErrorCode.MSP_ERROR_INVALID_DATA;
            }

            if (buffer != null && buffer.length < (length + offset)) {
                DebugLog.LogE("writeAudio error, buffer length < offset + length.");
                return ErrorCode.MSP_ERROR_INVALID_DATA;
            }

            ((MfvVerifier) mscer).writeData(ssub, params, buffer, offset, length);
            
            return ErrorCode.SUCCESS;
        }
    }

    public void stopWrite(String ssub) {
        synchronized (mSynObj) {
            if (mscer != null) {
                ((MfvVerifier) mscer).stopWrite(ssub, true);
            }
        }
    }
    
    public int execute(String ssub, String cmd, String params, final IdentityListener listener) {
    	int ret = ErrorCode.SUCCESS;
    	
    	setParameter(SpeechConstant.MFV_SST, cmd);
    	ret = startWorking(listener);
    	if( ErrorCode.SUCCESS==ret ){
    		ret = writeData(ssub, params, null, 0, 0);
    		stopWrite(ssub);
    	}
    	
    	return ret;
    }

    /**
     * 取消消息，保存底层事件
     */
    @Override
    public void cancel(final boolean notifyError) {
        // 识别会话取消，后台音乐继续播放.
        FuncAdapter.UnLock(mContext, mRequestFocus, null);
        super.cancel(notifyError);
    }

    private final class InternalListener implements IdentityListener {
        private IdentityListener mOutListener     = null;
        private static final int   MSG_ERROR        = 0;
        private static final int   MSG_AUDIO_BUFFER = 1;
        private static final int   MSG_SPEECH_BEGIN = 2;
        private static final int   MSG_SPEECH_END   = 3;
        private static final int   MSG_RESULT       = 4;
        // EVENT自定义消息
        private static final int   MSG_EVENT        = 6;
        private boolean            isFirstResult    = false;

        public InternalListener(final IdentityListener outlistener) {
            mOutListener = outlistener;
        }

        private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
        	@Override
        	public void handleMessage(
        			final Message msg) {
        		if (mOutListener == null) {
        			return;
        		}
        		switch (msg.what) {
        		case MSG_ERROR:
        			mOutListener.onError((SpeechError) msg.obj);
        			break;
        		case MSG_AUDIO_BUFFER:
        			break;
        		case MSG_SPEECH_BEGIN:
        			break;
        		case MSG_SPEECH_END:
        			break;
        		case MSG_RESULT:
        			mOutListener.onResult((IdentityResult) msg.obj, 
        					msg.arg1 == 1 ? true : false);
        			if (false == isFirstResult) {
        				pushSessionInfo(SessPerflog.KEY_UI_FRS);
        				isFirstResult = true;
        			}
        			if (1 == msg.arg1) {
        				pushSessionInfo(SessPerflog.KEY_UI_LRS);
        			}
        			break;
        		case MSG_EVENT:
        			Message mbody = (Message) msg.obj;
        			// 自定义消息数据
        			mOutListener.onEvent(mbody.what,
        					mbody.arg1, mbody.arg2,(Bundle)mbody.obj);
        			break;
        		default:
        			break;
        		}
        		super.handleMessage(msg);
        	}
        };

        @Override
        public void onResult(final IdentityResult result, final boolean isLast) {
            // 当返回最后一个结果，判断是否需要保存音频或继续音乐播放
            if (isLast) {
                sessionEnd();
            }
            Message msg = mUiHandler.obtainMessage(MSG_RESULT,
                    isLast == true ? 1 : 0, 0, result);
            mUiHandler.sendMessage(msg);
        }

        @Override
        public void onError(final SpeechError error) {
            sessionEnd();
            Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
            mUiHandler.sendMessage(msg);
        }

        @Override
        public void onEvent(final int eventType, final int arg1,
                final int arg2, final Bundle obj) {

            Message msgbody = new Message();
            msgbody.what = eventType;
            msgbody.arg1 = arg1;
            msgbody.arg2 = arg2;
            msgbody.obj = obj;

            Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0, 0, msgbody);
            mUiHandler.sendMessage(tmsg);
        }

        /**
         * 会话结束，判断是否需要保存音频或继续音乐播放，该接口不对外开放。
         */
        protected void sessionEnd() {
           ((MfvVerifier) mscer).getBuffer().saveToFile();
            // 识别会话结束，后台音乐继续播放.
            FuncAdapter.UnLock(mContext, mRequestFocus, null);
        }
    }

    public boolean isWorking() {
        return isRunning();
    }
    
}
