/*
 * 文件名：SpeechUnderstander.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：语义理解对象；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.speech.impl;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.RecognizerListener;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechListener;
import com.iflytek.cloud.SpeechUnderstanderListener;
import com.iflytek.cloud.UnderstanderResult;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.msc.MSC;
/**
 * 语音转语义类。实例化该对象之前需要先调用{@link 
 * com.iflytek.cloud.SpeechUtility#createUtility(Context, String)}
 * 进行初始化。<br>
 * @author <a href="http://www.xfyun.cn/">讯飞语音云</a>
 *
 */
public class SpeechUnderstanderImpl{
	//语义理解对象.
	protected static SpeechUnderstanderImpl mInstance = null;
	
	// 识别代理对象
	private SpeechRecognizerImpl  mRecognizer = null;
	/**
	 * 初始化语义识别对象
	 * @param context 上下文环境
	 */
	public SpeechUnderstanderImpl(Context context) {
		mRecognizer = new SpeechRecognizerImpl(context);

	}

	/**
	 * 创建识别对象，多次调用不会重复初始化
	 * @param context 上下文环境.
	 * @return 语义识别对象.
	 */
	public synchronized static SpeechUnderstanderImpl createUnderstander(Context context) {
		if (mInstance == null) {
			mInstance = new SpeechUnderstanderImpl(context);
		}
		return mInstance;
	}
	
	/**
	 * 获得语义识别对象
	 * @return 语义识别对象
	 */
	public static SpeechUnderstanderImpl getUnderstander() {
		return mInstance;
	}


	/**
	 * 开始语义识别，可以通过{@link #setParameter(String, String)}的AUDIO_SOURCE
	 * 字段来设置是否需要通过麦克进行录音。<br>
	 * @param listener 回调监听
	 * @return void
	 */
	public int startUnderstanding(SpeechUnderstanderListener listener) {
		SelfRecognitionListener recoListener = new SelfRecognitionListener(listener);
		
		if(TextUtils.isEmpty(mRecognizer.getParameter(SpeechConstant.ASR_SCH))) 
			mRecognizer.setParameter(SpeechConstant.ASR_SCH, "1");
		if(TextUtils.isEmpty(mRecognizer.getParameter(SpeechConstant.NLP_VERSION))){
			final String defNlpVersion = MSC.isIflyVersion() ? "3.0" : "2.0";
			mRecognizer.setParameter(SpeechConstant.NLP_VERSION, defNlpVersion);
		}
		if(TextUtils.isEmpty(mRecognizer.getParameter(SpeechConstant.RESULT_TYPE)))
			mRecognizer.setParameter(SpeechConstant.RESULT_TYPE, "json");
		
		mRecognizer.startListening(recoListener);
		return ErrorCode.SUCCESS;
	}

	/**
	 * 是否正在进行识别
	 */
	public boolean isUnderstanding() {
		return mRecognizer.isListening();
	}

	/**
	 * 写入录音数据。<br>
	 * @param buffer 写入的录音数据<br>
	 * @param offset 录音数据起始位置<br>
	 * @param length 录音数据长度<br>
	 * @return 是否成功写入数据
	 */
	public int writeAudio(byte[] buffer, int offset, int length) {
		return mRecognizer.writeAudio(buffer, offset, length);
	}

	/**
	 * 停止录音
	 */
	public void stopUnderstanding() {
		mRecognizer.stopListening();
	}
	
	/**
	 * 取消识别
	 */
	public void cancel(boolean notifyError) {
		mRecognizer.cancel(notifyError);
	}
	
	/**
	 * 语义识别参数设置，key可设置的参数有：<br>
	 * SpeechConstant.NET_TYPE:网络类型 <br>
	 * SpeechConstant.NET_TIMEOUT:网络连接超时时间 <br>
	 * SpeechConstant.KEY_SPEECH_TIMEOUT:语音输入超时时间 <br>
	 * SpeechConstant.LANGUAGE:语言<br>
	 * SpeechConstant.ACCENT:语言区域<br>
	 * SpeechConstant.DOMAIN:应用领域<br>
	 * SpeechConstant.CLOUD_GRAMMAR:云端语法ID<br>
	 * SpeechConstant.AUDIO_SOURCE:音频源<br>
	 * SpeechConstant.VAD_BOS:前端点超时<br>
	 * SpeechConstant.VAD_EOS:后端点超时<br>
	 * SpeechConstant.SAMPLE_RATE:识别采样率<br>
	 * SpeechConstant.ASR_NBEST:句子级多候选<br>
	 * SpeechConstant.ASR_WBEST:词级多候选<br>
	 * SpeechConstant.ASR_PTT:设置是否有标点符号<br>
	 * SpeechConstant.RESULT_TYPE:识别结果类型<br>
	 * SpeechConstant.SEARCH_AREA:地图搜索设置区域<br>
	 * SpeechConstant.ASR_AUDIO_PATH:识别录音保存路径
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return 错误码，0表示成功
	 */
	public boolean setParameter(String key, String value) {
		return mRecognizer.setParameter(key, value);
	}
	
	/**
	 * 获取语义参数。
	 * <p>包括引擎类型、语言、语言区域、领域、前后端点超时等，另外包括支持的语言列表等。
	 * @param key 参数名称
	 * @return 参数值
	 */
	public String getParameter(String key) {
		return mRecognizer.getParameter(key);
	}

	public boolean setParameter(HashParam params)
	{
		return mRecognizer.setParameter(params);
	}
	
	/**
	 * 销毁识别对象
	 */
	public boolean destroy() {
		boolean ret = mRecognizer.destroy();
		if(ret)
			mInstance = null;
		return ret;
	}
	
	 /**
     * 内部识别的回调
     */
    private class SelfRecognitionListener implements RecognizerListener {
        
        private final SpeechUnderstanderListener mInterListener;

        public SelfRecognitionListener(SpeechUnderstanderListener listener) {
        	mInterListener = listener;
        }
        
        @Override
        public void onEndOfSpeech() {
        	if(mInterListener != null)
        		mInterListener.onEndOfSpeech();
        }

        @Override
        public void onVolumeChanged(int volume , byte[] data) {
        	if(mInterListener != null)
        		mInterListener.onVolumeChanged(volume,data);
        }
        
		@Override
		public void onBeginOfSpeech() {
			if(mInterListener != null)
        		mInterListener.onBeginOfSpeech();
		}

		@Override
		public void onError(SpeechError error) {			
			if(mInterListener != null && error != null)
        		mInterListener.onError(error);
		}
		
		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			if(mInterListener != null)
        		mInterListener.onEvent(eventType, arg1,arg2, obj);	
		}
		
		@Override
		public void onResult(com.iflytek.cloud.RecognizerResult result,
				boolean islast) {
			if(mInterListener != null)
        		mInterListener.onResult(new UnderstanderResult(
        				result.getResultString()));
                   
		}
    };
}