package com.iflytek.cloud.util;

import com.iflytek.cloud.util.code.impl.AudioCoderImpl;

import android.content.Context;

/**
 * <h3>编解码类</h3>
 * <p>
 * 		通过编解码类，实现音频的编解码。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createCoder}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getCoder}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createCoder}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link com.iflytek.cloud.SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1083
 * @author iFlytek &nbsp;&nbsp;&nbsp;<a href="http://www.xfyun.cn/">
 * 				讯飞语音云</a>
 * 
 * @see com.iflytek.cloud.SpeechUtility
 */
public abstract class AudioCoder {
	/**
	 * <h3>结果类</h3>
	 * <p>
	 * 		在编解码时，编解码的结果数据。
	 * </p>
	 */
	public static class CoderResult{
		/**
		 * <h4>音频数据缓存</h4>
		 * <p>
		 * 		存放音频数据的内存空间。
		 * </p>
		 * <p>值类型：byte[]</p>
		 * <p>值域：{null, 对应的数据数组}</p>
		 */
		public  byte[] buffer = null;
		
		/**
		 * <h4>数据偏移</h4>
		 * <p>
		 * 		指定{@link #dataBuf}有效数据开始位置，相对当前数组开始位置的偏移大小。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, +∞]</p>
		 */
		public int offset = 0;
		
		/**
		 * <h4>数据长度</h4>
		 * <p>
		 * 		数据长度，指定{@link #dataBuf}中有效的数据的长度。
		 * bufLen <= dataBuf.length - offset。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, +∞]</p>
		 */
		public int length = 0;
		
		/**
		 * <h4>错误信息</h4>
		 * <p>
		 * 		当此值不为0时，表示出现错误。
		 * </p>
		 * 
		 * <p>值类型：int</p>
		 * <p>值域：(-∞, +∞)</p>
		 * 
		 * @see com.iflytek.cloud.ErrorCode
		 */
		public int error = 0;
		
	}//end of class Keys
	
	/**
	 * <h4>编解码库</h4>
	 * <p>
	 * 		编解码库指编码或解码时，用到的库，目前支持以下编解码库：
	 * <br>speex。
	 * <br>在编码时，可以指定任意支持的编码库，但在解码时，必须指定待解码音频格式
	 * 一致的的解码库，否则解码失败，或结果音频与预想的不一致。
	 * </p>
	 * 
	 * <p>是否必须设置：是</p>
	 * <p>默认值："speex"</p>
	 * <p>值范围：{"speex"}</p>
	 */
	public static final String LIBRARY = "lib";
	
	/**
	 * <h4>编码等级</h4>
	 * <p>
	 * 		编码等级，指编码时，压缩的等级。解码时不必指定。
	 * <br>speex：[0, 10]，默认0;
	 * </p>
	 * 
	 * <p>是否必须设置：是（编码时）</p>
	 * <p>默认值：0</p>
	 * <p>值范围：[0, 10]</p>
	 */
	public static final String LEVEL = "level";
	
	/**
	 * <h4>编码模式</h4>
	 * <p>
	 * 		是否是宽带模式（仅在speex库中支持）。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{0, 1}</p>
	 */
	public static final String MODE = "mode";
	
	/**
	 * <h4>常量值：最大缓存大小</h4>
	 * <p>
	 * 		编解码音频时，应用层一次传入音频的最大字节大小。
	 * 当一次传入最大音频字节大于此值时，可能导致编解码失败。
	 * 同时，编解码后返回的音频可能会大于或小于此值。
	 * </p>
	 * @see #encode
	 * @see #decode
	 */
	public final static int MAX_BUF_LEN = 1024 * 32;
	
	protected static AudioCoder sInstance = null;
	protected static Object sSyncObj = new Object();
	
	protected AudioCoder( Context context, String param ){}
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getCoder}获取已创建的单例对象。
	 * </p>
	 * 
	 * @param context 应用上下文
	 * @param param 初始化参数 包括：
	 * <br>{@link #LIBRARY}：编码库;
	 * 	<br>{@link #MODE}：编码模式；
	 * 
	 * @return 编解码对象
	 * @see #destroy
	 * @see #getCoder
	 */
	public static AudioCoder createCoder( Context context, String param ){
		synchronized( sSyncObj ){
			if( null == sInstance ){
				sInstance = AudioCoderImpl.createCoder( context, param );
			}
		}//end of synchronized
		
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createCoder}创建单例对象。
	 * </p>
	 * 
	 * @return 编解码对象
	 * 
	 * @see #createCoder
	 * @see #destroy
	 */
	public static AudioCoder getCoder(){
		synchronized( sSyncObj ){
			// synchronized with createCoder
		}
		
		return sInstance;
	}
	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createCoder}创建的单例对象。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createCoder}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createCoder
	 */
	abstract public boolean destroy();
	
	/**
	 *<h4>编码</h4>
	 *<p>
	 *		通过在参数中，填充待编码音频数据，及其他相关的数据信息。参考{@link CoderResult}。
	 *</p>
	 *<p>
	 * 		每次传入的数据长度，应为1帧音频大小的整数倍，如音频为16位、单声道的Windows PCM音频时，则
	 * 数据长度应为 16/8=2字节的整数倍。
	 *</p>
	 *<p>
	 *		此函数将把音频数据编码后，同步通过返回值返回，将阻塞当前线程。
	 *</p>
	 * @param buffer 待编码音频缓存区
	 * @param offset 有效音频开始位置编移
	 * @param length 有效音频数据长度
	 * @param isLast 是否是最后一段音频
	 * @return 已编码数据
	 */
	abstract public CoderResult encode( byte[] buffer, int offset, int length, boolean isLast );
	
	/**
	 * <h4>解码</h4>
	 * 	<p>
	 *		通过在参数中，填充待解码音频数据，及其他相关的数据信息。参考{@link CoderResult}。
	 *	</p>
	 *<p>
	 * 		每次传入的数据长度，应为1帧音频大小的整数倍。
	 *</p>
	 *	<p>
	 *		此函数将把音频数据解码后，同步通过返回值返回，将阻塞当前线程。
	 *	</p>
	 * @param buffer 待解码音频缓存区
	 * @param offset 有效音频开始位置编移
	 * @param length 有效音频数据长度，单次写入最大长度限制，见{@link #MAX_BUF_LEN}。
	 * @param isLast 是否是最后一段音频
	 * @return 已解码数据
	 */
	abstract public CoderResult decode( byte[] buffer, int offset, int length, boolean isLast );
	
	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		在编解码前，可设置编解码的参数，包括：
	 * <br>{@link #LEVEL}：编码等级；
	 * </p>
	 * @param key 参数名
	 * @param value 参数值
	 */
	abstract public void setParameter( String key, String value );
}
