package com.iflytek.cloud.util;

import java.io.File;

import com.iflytek.cloud.msc.util.log.DebugLog;

import android.content.Context;
import android.content.res.AssetFileDescriptor;
import android.text.TextUtils;

/**
 * <h3>资源辅助类</h3>
 * <p>
 * 		资源辅助类，将资源目录，转换为SDK要求的目录形式。
 * </p>
 * <p>
 * 		SDK要求的文件目录格式如下（由此类的函数生成，不关心的用户可忽略），两个文件间，
 * 以英文分号";"分隔：
 * <br>fd|file_info|offset|length;
 * <br>fo|file_info|offset|length;
 * <br>其中fd表示文件标识符，通过assets、res读取时传递，fo表示文件路径方式。
 * <br>注意：每次通过{@link #generateResourcePath}生成assets、res的文件路径后，都会生
 * 成一个文件句柄，为了使句柄释放，需要通过对应的设置资源路径函数传给SDK，用于使用或释放，
 * 以免造成内存泄露。关于资源路径设置，请参考具体的业务类说明。
 * </p>
 * <p>
 * 		SDK加载离线资源有三种方式：
 * </p>
 * <p>
 * 		1，通过{@link com.iflytek.cloud.SpeechUtility#createUtility}初始化SDK时加载，通过
 * 设置{@link #ENGINE_START}参数，并根据业务不同，指定以下一个或多个的资源路径参数： 
 * <br>{@link #TTS_RES_PATH}，
 * <br>{@link #ASR_RES_PATH}，
 * <br>{@link com.iflytek.cloud.SpeechConstant#IVW_RES_PATH}，
 * <br>{@link com.iflytek.cloud.SpeechConstant#IVW_ENROLL_RES_PATH}。
 * </p>
 * <p>
 * 		2，通过{@link com.iflytek.cloud.SpeechUtility#setParameter(String, String)}加载，通过
 * 设置{@link #ENGINE_START}参数，并根据业务不同，指定以下一个或多个的资源路径参数： 
 * <br>{@link #TTS_RES_PATH}，
 * <br>{@link #ASR_RES_PATH}，
 * <br>{@link com.iflytek.cloud.SpeechConstant#IVW_RES_PATH}，
 * <br>{@link com.iflytek.cloud.SpeechConstant#IVW_ENROLL_RES_PATH}。
 * </p>
 * <p>
 * 		3，通过各业务开始会话的函数加载，通过各业务的setParameter函数，如
 * {@link com.iflytek.cloud.SpeechRecognizer#setParameter(String, String)}，根据业务不同，
 * 设置以下资源路径参数中的一个：
 * <br>{@link #TTS_RES_PATH}，
 * <br>{@link #ASR_RES_PATH}，
 * <br>{@link com.iflytek.cloud.SpeechConstant#IVW_RES_PATH}，
 * <br>{@link com.iflytek.cloud.SpeechConstant#IVW_ENROLL_RES_PATH}。
 * </p>
 * <p>
 * 		在同一业务，新的资源加载后，上一个资源将自动被释放。如合成中，如果各发音人资源文件是
 * 分开的，则在加载当前发音人资源后，上一发音人资源将自动被释放。如果要释放当前的资源，
 * 而不加载新的资源，需要同时销毁离线引擎，有以下三种方式：
 * </p>
 * <p>
 * 		1，通过{@link com.iflytek.cloud.SpeechUtility#setParameter(String, String)}，设置参数
 * {@link #ENGINE_DESTROY}即可。
 * </p>
 * <p>
 * 		2，通过各业务的destory函数销毁：
 * <br>{@link com.iflytek.cloud.SpeechRecognizer#destroy()}，
 * <br>{@link com.iflytek.cloud.SpeechSynthesizer#destroy()}，
 * <br>{@link com.iflytek.cloud.VoiceWakeuper#destroy()}。
 * </p>
 * <p>
 * 		3，通过{@link com.iflytek.cloud.SpeechUtility#destroy()}销毁。
 * </p>
 * <p>
 * 		另外，在应用退出后，所有应用相关的资源会被销毁。因为销毁引擎后，重新启动引擎的时间要比
 * 直接更换资源长（会使首次会话的时间变长），所以如果仅是短暂的停止使用，建议不需要销毁资源。
 * </p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * 
 * @see com.iflytek.cloud.SpeechConstant#ENGINE_TYPE
 * @see com.iflytek.cloud.SpeechConstant#IVW_RES_PATH
 * @see com.iflytek.cloud.SpeechConstant#IVW_ENROLL_RES_PATH
 * @see com.iflytek.cloud.SpeechUtility#createUtility
 * @see com.iflytek.cloud.SpeechUtility#setParameter
 * @see com.iflytek.cloud.SpeechRecognizer#setParameter(String, String)
 * @see com.iflytek.cloud.SpeechSynthesizer#setParameter(String, String)
 * @see com.iflytek.cloud.VoiceWakeuper#setParameter(String, String)
 */
public class ResourceUtil {

	/**
	 * <h4>启动引擎</h4>
	 * <p>
	 * 		通过设置此参数，启动离线引擎。在离线功能使用时，首次设置资源路径时，需要设置此参
	 * 数启动引擎。关于设置的方式，参考类说明。
	 * </p>
	 * <p>
	 * 		启动引擎参数值因业务类型而异：
	 * <br>合成：{@link com.iflytek.cloud.SpeechConstant#ENG_TTS}
	 * <br>识别：{@link com.iflytek.cloud.SpeechConstant#ENG_ASR}
	 * <br>唤醒：{@link com.iflytek.cloud.SpeechConstant#ENG_IVW}
	 * </p>
	 * 
	 * <p>是否必须设置：是（在首次使用离线功能时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：见上文</p>
	 * 
	 * @see #ENGINE_DESTROY
	 */
	public static final String ENGINE_START = "engine_start";
	
	/**
	 * <h4>销毁引擎</h4>
	 * <p>
	 * 		通过调用{@link com.iflytek.cloud.SpeechUtility#setParameter(String, String)}设置此参数
	 * 以销毁引擎，并释放资源内存。(通过destory函数销毁引擎时，则不需要设置此参数，参考类说明)
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：参考{@link #ENGINE_START}说明</p>
	 * 
	 * @see #ENGINE_START
	 */
	public static final String ENGINE_DESTROY = "engine_destroy";
	
	/**
	 * <h4>识别资源路径</h4>
	 * <p>
	 * 		在离线识别时，首次会话，或切换识别资源时，需要设置识别资源路径。
	 * 资源路径值需要通过{@link #generateResourcePath}生成SDK要求的格式。
	 * </p>
	 * 
	 * <p>是否必须设置：是（离线识别时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：见上文说明</p>
	 * 
	 * @see #TTS_RES_PATH
	 */
	public static final String ASR_RES_PATH = "asr_res_path";
	
	/**
	 * <h4>语法构建目录</h4>
	 * <p>
	 * 		在使用离线语法时，需要构建语法并保存到本地，在构建和使用语法时，都需要设置语法的构
	 * 建目录。与识别资源路径值不一样的是，语法的路径值，不需要通过{@link #generateResourcePath}
	 * 生成SDK要求的格式。
	 * </p>
	 * 
	 * <p>是否必须设置：是（离线语法识别时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件夹径值（含文件名）</p>
	 */
	public static final String GRM_BUILD_PATH = "grm_build_path";
	
	/**
	 * <h4>合成资源路径</h4>
	 * <p>
	 * 		在离线合成时，首次会话，或切换合成资源时，需要设置资源路径。
	 * 资源路径值需要通过{@link #generateResourcePath}生成SDK要求的格式。
	 * 同时，合成设置资源路径时，必须同时设置发音人参数
	 * {@link com.iflytek.cloud.SpeechConstant#VOICE_NAME}，值为资源中包含的
	 * 其中一个发音人名参数。
	 * </p>
	 * 
	 * <p>是否必须设置：是（离线合成时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：见上文说明</p>
	 * 
	 * @see #ASR_RES_PATH
	 */
	public static final String TTS_RES_PATH = "tts_res_path";

	/**
	 * <h4>唤醒资源路径</h4>
	 * <p>
	 * 		唤醒需要使用本地资源，通过此参数设置本地资源所在的路径。多个资源间，以英文分号";"分隔。
	 * 与{@link com.iflytek.cloud.SpeechConstant#IVW_ENROLL_RES_PATH}一样，用于使用的资源，需要通过
	 * {@link #generateResourcePath}生成标准的资源路径值。
	 * 请参考 {@link #generateResourcePath}。
	 * </p>
	 * 
	 * <p>是否必须设置：是（在非注册时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的资源文件路径</p>
	 *  
	 * @see com.iflytek.cloud.SpeechConstant#IVW_SST
	 * @see com.iflytek.cloud.VoiceWakeuper#setParameter(String, String)
	 * //@see com.iflytek.cloud.VoiceWakeuper#updateWords(String, String)
	 */
	public static final String IVW_RES_PATH = "ivw_res_path";

	/**
	 * <h3>资源路径类型</h3>
	 * <p>
	 *		资源路径类型，即资源在移动设备（手机或平板）中存放位置的类型，在安卓平台上，
	 * 包括：
	 * <br>APK包的assets目录：assets；
	 * <br>APK包的res目录：res；
	 * <br>SD卡的目录：path；
	 * </p>
	 * <p>
	 * 		通过{@link ResourceUtil#generateResourcePath}生成路径时，根据类型不同，
	 * 在路径参数中，传入对应的值：
	 * 		<br>assets类型时，若资源存放位置为/assets/resource.jet，则在路径参数传入值为
	 * "resource.jet";
	 * 		<br>res类型时，通过R类获取ID，并转换为{@link java.lang.String}类型，如存放位
	 * 置为/res/raw/resource.jet，则在路径参数传入值为String.valueOf(R.raw.resource)；
	 * 		<br>path类型时，直接传入对应的路径值，如存放位置为/sdcard/resource.jet，则
	 * 在路径参数传入值为"/sdcard/resource.jet"；
	 * </p>
	 * 
	 * @see ResourceUtil#generateResourcePath
	 */
	public enum RESOURCE_TYPE 
	{
		assets,
		res,
		path
	}
	
	/**
	 * <h4>生成资源路径</h4>
	 * <p>
	 * 		传入不同类型的路径值，生成SDK要求的路径值，用于各业务离线功能时，传入资源路径。
	 * </p>
	 * 
	 * @param context 应用上下文
	 * @param type 资源路径类型
	 * @param path 原资源路径
	 * 
	 * @return SDK要求的资源路径值
	 */
	public static String generateResourcePath(Context context,RESOURCE_TYPE type,String path)
	{
		String ret = null;
		if(type == RESOURCE_TYPE.path)// path绝对路径读取
			ret = getAbsolutePath(path);
		else // assets、res目录下读取
			ret = getAssetOrResPath(context,type, path);
		return ret;
	}
	
	/**
	 * 返回assets目录下的资源路径
	 * @param context
	 * @param type
	 * @param id
	 * @return
	 */
	private static String getAssetOrResPath(Context context,RESOURCE_TYPE type,String id)
	{
		if(TextUtils.isEmpty(id) || context == null)
			return null;
		String path = context.getPackageResourcePath();
		AssetFileDescriptor des = null;
		String ret = null;
		long offset = 0;
		long length = 0;
		try {
			if(type == RESOURCE_TYPE.assets)	// assets
			{
				des = context.getAssets().openFd(id);
				offset = des.getStartOffset();
				length = des.getLength();
			}
			else								// res
			{
				int resid = Integer.valueOf(id);
				des = context.getResources().openRawResourceFd(resid);
				offset = des.getStartOffset();
				length = des.getLength();
			}
			
			// 需要在MSC JNI中增加获取fd的方法
			ret = "fo|"+path+"|"+offset+"|"+length;
		} catch (Exception e) {
			DebugLog.LogE(e);
		}finally
		{
			try {
				if(des != null)
				{
					des.close();
					des = null;
				}
			} catch (Exception e2) {
				e2.printStackTrace();
			}
		}
		return ret;
	}
	
//	/**
//	 * 返回assets目录下的资源路径
//	 * @param context
//	 * @param path
//	 * @return
//	 */
//	private static String getAssetOrResPath(Context context,RESOURCE_TYPE type,String id)
//	{
//		if(TextUtils.isEmpty(id) || context == null)
//			return null;
//		String ret = null;
//		AssetManager assetmanager = null;
//		AssetFileDescriptor filefd = null;
//		try {
//			if(type == RESOURCE_TYPE.assets)	// assets
//			{
//				assetmanager = context.getAssets();
//				filefd = assetmanager.openFd(id);
//				
//			}
//			else								// res
//			{
//				int resid = Integer.valueOf(id);
//				filefd = context.getResources().openRawResourceFd(resid);
//			}
//			
//			long offset = filefd.getStartOffset();
//			long length = filefd.getLength();
//			// 需要在MSC JNI中增加获取fd的方法
//			if(MSC.isLoaded())
//			{
//				int fd = MSC.getFileDescriptorFD(filefd.getFileDescriptor());
//				if(fd != -1)
//					ret = "fd|"+fd+"|"+offset+"|"+length;
//			}
//		} catch (Exception e) {
//			DebugLog.LogE(e);
//		}finally
//		{
//			try {
//				if(filefd != null)
//					filefd.close();	
//				filefd = null;
//			} catch (Exception e2) {
//				e2.printStackTrace();
//			}
//		}
//		return ret;
//	}
	
	/**
	 * 返回绝对路径下的资源路径
	 * @param path
	 * @return
	 */
	private static String getAbsolutePath(String path)
	{
		if(TextUtils.isEmpty(path))
			return null;
		
		File f = new File(path);
		if(!f.exists() || !f.isFile())
			return null;
		long length = f.length();
		long offset = 0;
		return "fo|"+path+"|"+offset+"|"+length;
	}
	
	
	
	/////////////////////////////////////个性化数据上传///////////////////////////////////////
}
