
package com.iflytek.cloud.util.contacts;

import android.content.Context;
import android.database.ContentObserver;
import android.net.Uri;
import android.os.Handler;
import android.os.HandlerThread;

import java.util.ArrayList;
import java.util.List;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.util.ContactManager;
import com.iflytek.cloud.util.contacts.entities.ContactSet;
import com.iflytek.cloud.util.contacts.impl.ContactAccessorSdk3_4;
import com.iflytek.cloud.util.contacts.impl.ContactAccessorSdk5;
import com.iflytek.cloud.util.contacts.interfaces.ContactAccessor;


/**
 * 联系人管理类，用来获取联系人姓名等数据<br>
 * 格式：<br>
 * &nbsp;&nbsp;&nbsp;张三<br>&nbsp;&nbsp;&nbsp;李四<br>&nbsp;&nbsp;&nbsp;王五
 * @author iFlytek &nbsp;&nbsp;&nbsp;<a href="http://www.xfyun.cn/">讯飞语音云</a>
 */

public class ContactManagerImpl extends ContactManager{
    
	public static final String TAG = "iFly_ContactManager";


/**
	 * 联系人的名称文件
	 */

    private static final String FILE_NAME_CACHE = "name.txt"; 
    
    private static final String WORK_THREAD = "ContactManager_worker";
    
    private static ContactManagerImpl mInstance = null;
    
    private static Context mContext = null;
    
    private static int SDK_DONUT = 4;
    
    private static ContactAccessor mContactAccessor = null;
    
    private static ContactHelper mContactHelper = null;
    
    private static ContactObserver mContactObserver;
    
    private HandlerThread sWorkerThread = null;
    
    private static ContactListener mContactListener = null;
    
    private Handler sWorkerQueue;
    
    private static final char SPLIT_CHAR = '\n';
    
    private static final char LOG_CHAR = '&';

/**
     * 获取ContactManager的实例
     * @return ContactManager的实例
     */

    public static ContactManagerImpl getManager() {
        return mInstance;
    }
    

/**
     * 创建ContactManager对象，注册联系人监听和联系人名称查询回调
     * @param context
     * @return ContactManager对象
     */

    public static ContactManagerImpl createManager(Context context, ContactListener contactListener) {
        mContactListener = contactListener;
        mContext = context;
        if (null == mInstance) {
            mInstance = new ContactManagerImpl();
            mContext.getContentResolver().registerContentObserver(mContactAccessor.getContactUri(), true, mContactObserver);
        }
        return mInstance;
    }
    
    private ContactManagerImpl() {
        if (android.os.Build.VERSION.SDK_INT > SDK_DONUT) {
            mContactAccessor = new ContactAccessorSdk5(mContext);
        } else {
            mContactAccessor = new ContactAccessorSdk3_4(mContext);
        }
        
        mContactHelper = new ContactHelper(mContext, mContactAccessor);
        sWorkerThread = new HandlerThread(WORK_THREAD);
        sWorkerThread.start();
        sWorkerQueue = new Handler(sWorkerThread.getLooper());
        sWorkerThread.setPriority(Thread.MIN_PRIORITY);

        mContactObserver = new ContactObserver(sWorkerQueue);
    }
    

/**
     * 同步查询所有联系人名称
     * @return 联系人名称
     */

    public String queryAllContactsName() {
        if (null != mContactHelper) {
        	StringBuilder sb = new StringBuilder();
        	String[] strs = mContactHelper.getAllContactNames();
        	for(String str : strs)
        		sb.append(str + SPLIT_CHAR);
            return sb.toString();
        }
        
        return null;
    }
    

/**
     * 启动线程查询所有联系人名称
     */

    public void asyncQueryAllContactsName() {
        sWorkerQueue.post(new Runnable() {
            @Override
            public void run() {
                queryAllContactsNameImpl();
            }
        });
    }
    
    private void queryAllContactsNameImpl() {       
    	try {
    		if (null != mContactListener && null != mContactHelper) {
            	String[] contactNames = mContactHelper.getAllContactNames();
            	String newNameString = PhoneNumberUtil.mergeStrings(contactNames, SPLIT_CHAR);
            	String nameFile = mContext.getFilesDir().getParent() + '/' + FILE_NAME_CACHE;
            	String oldNameString = FileManager.readString(nameFile);
            	if (null != newNameString && null != oldNameString && newNameString.equals(oldNameString)) {
            		DebugLog.LogD(ContactManagerImpl.TAG, "contact name is not change.");
                    mContactListener.onContactQueryFinish(newNameString, false);
                } else {
                	FileManager.writeString(nameFile, newNameString, true);
                	mContactListener.onContactQueryFinish(newNameString, true);
                }
            }
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
    }
    

/**{@hide}*/

    public String queryAllContactsInfoImpl() {
    	if (null != mContactHelper) {
    		String[] contactInfos = mContactHelper.getAllContactInfo();
        	String newInfoString = PhoneNumberUtil.mergeStrings(contactInfos, LOG_CHAR);
        	return newInfoString;
        }
    	return null;
    }
    

/**
     * 重新构建联系人词典，查询联系人
     */
    public void reInitQueryContact() {
        sWorkerQueue.post(new Runnable() {
            @Override
            public void run() {
                queryAllContactsNameImpl();
            }
        });
    }
    

/**
     * 清除联系人相关缓存
     */

    public static void deleteCacheFile(Context ctx) {
        if (null != ctx){
            String nameFile = ctx.getFilesDir().getParent() 
            	+ '/' + FILE_NAME_CACHE;
            FileManager.deleteFileFromPath(nameFile);
            DebugLog.LogD(ContactManagerImpl.TAG, "deleteCacheFile " + nameFile);
        }
    }
    

/**
     * 根据号码查询联系人ID
     * @param number 联系人号码
     * @return 联系人ID
     */

    public String queryContactId(String number) {
        if (null != mContactAccessor) {
            return mContactAccessor.queryContactIdByNumber(number);
        }
        
        return null;
    }
    

/**
     * 根据号码获取联系人名称
     * @param number 联系人号码
     * @return 联系人名称
     */

    public String queryContactNameByNumber(String number) {
        if (null != mContactHelper) {
            return mContactHelper.getContactNameByNumber(number);
        }
        
        return null;
    }
    

/*
     * 根据姓名获取相应的contactSet信息
     * @param names 联系人号码集合
     * @param fuzzyFlag 是否模糊查询的标志， true表示模糊查询， false表示严格按照姓名查询
     * @return contactSet信息
     */

	public ArrayList<ContactSet> queryContactSetByNames(boolean fuzzyFlag, List<String> names) {
	    if (null != mContactHelper) {
	        return mContactHelper.queryContactSetByNames(fuzzyFlag, names);
	    }
	    return null;
	}
	

/**
	 * 根据姓名判断联系人是否存在号码
	 * @param name 联系人姓名
	 * @return 返回true表示存在
	 */

	public boolean hasContactNumber(String name) {
	    if (null != mContactHelper) {
	        return mContactHelper.hasPhoneNumber(name);
	    }
	    return false;
	}
	

/**
	 * 根据姓名获取联系人头像信息
	 * @param name 联系人姓名
	 * @return 联系人头像信息
	 */

	public byte[] queryContactPhoto(String name) {
		 if (null != mContactHelper) {
			 return mContactHelper.queryContactPhotoData(name);
		 }
		 return null;
	}
	

/**
     *  获取联系人uri
     * @return 联系人uri
     */

    public Uri getContactUri() {
        if (null != mContactAccessor) {
            return mContactAccessor.getContactUri();
        }
        
        return null;
    }
    
    public void unregisterObserver() {
        if (null != mContactObserver) {
            mContext.getContentResolver().unregisterContentObserver(mContactObserver);
            if(sWorkerThread != null)
            	sWorkerThread.quit();
        }
    }

/**
     * 根据姓名和号码新建联系人
     * @param name 联系人姓名
     * @param number 联系人号码
     */

    public void insertNewContacts(String name, String number) {
        if (null != mContactAccessor) {
            mContactAccessor.insertNewContact(name, number);
        }
    }

/**
     * 在通讯录中插入新的联系人
     * @param name 联系人姓名
     * @param number 联系人号码
     */

    public void insertIntentContact(String name, String number) {
    	 if (null != mContactAccessor) {
             mContactAccessor.insertIntentContact(name, number);
         }
    }
    
    private long mLastChangeTime = 0;
    private static final long WAIT_CHANGE_TIME = 1000 * 5;
    
    private class ContactObserver extends ContentObserver {     
        public ContactObserver(Handler handler) {
            super(handler);
        }

        @Override
        public void onChange(boolean selfChange) {
        	DebugLog.LogD(ContactManagerImpl.TAG, "ContactObserver_Contact| onChange");
            if ((System.currentTimeMillis() - mLastChangeTime < WAIT_CHANGE_TIME)) {
            	DebugLog.LogD(ContactManagerImpl.TAG, "onChange too much");
                return;
            }
            mLastChangeTime = System.currentTimeMillis();
            queryAllContactsNameImpl();
        }
    }
    
    public static void destroy(){
    	if(mInstance != null){
    		mInstance.unregisterObserver();
    		mInstance = null;
    	}
    }
}

